#!/bin/bash

declare VERSION="0.0.5d-jha"

# History (as always in the source code :-)
#
# 0.0.3	Seems to work fine - still lacks support for X x 432 Files
#	What about MPEG Files ? 
# 0.0.4	Improved getvidxy - No Temp Files needed anymore !
#
# 0.0.5b-jha
#       - Improved getvidxy - thanks to new mplayer option '-identify'
#       it gets now the correct values for X-/Y-resolution
#       and the framerate (if defined in the video file)
#       for all media files
#       - reworked the scaling calculation to scale only if needed
#       and if so, scale as little as possible to save CPU power
#       - distinguish between NTSC and PAL files; play them with
#       the right TV norm
#
# 0.0.5c-jha
#       - play videos with 24 or 23.976 fps as NTSC, not PAL
#
# 0.0.5d-jha
#       - added AC3 support (thanks to Christian Jacobsen
#       <christian.jacobsen -at- stageholding.de>)
#       - better support for mplayers SLAVE mode
#       - fixed some scaling problems
#       - some minor clean-ups
#


function readcfg () {

# Aufruf:
# readcfg CFGPARM
# Searches the config file for CFGPARM and the output is the configured value !
# If not found the output is empty (test -z to determine)

if test -z $1; then echolog "*** Internal Error - Called readcfg without Option Name"; return; fi


declare -a y[0]	# Anzahl der Zeilen (idealerweise = 1 !!!)
declare -i anzahl=`grep -v -e "#" $CFGFIL | grep -i -e $1 | wc -l`
if test "$anzahl" -eq 1;
  then y=(`grep -v -e "#" $CFGFIL | grep -i -e $1`)
  anzahl=${#y[*]} # Diesmal ANZAHL der Parameter pro Zeile !
  declare -i x=0
  while x=$x+1; test $x -lt $anzahl;
    do if ! test $x -eq $(($anzahl-1)) # Noch nicht letzter Wert der Zeile ?
    	then echo -n ${y[$x]}" ";	# Diese Ausgabe wird weitergegeben !
	else echo ${y[$x]};  # Letztes Argument !
       fi
  done;
  else logger "readcfg.sh error: Option $1 found $anzahl times in $CFGFIL";
fi

return;
}

function getvidxy () {

# Aufruf: getvidxy
# Ermittelt x und y Ausloesung des FILEs ! 
# Ausgabe: in Variable ORIG_X und ORIG_Y (global)

# Variablen Definitionen
local TEMP1


ORIG_X=0; ORIG_Y=0

TEMP1=`$MPLAYER -identify "$FILE" 2>&1 | grep -i -e "^ID_"`
debugmsg "OutputFromMPLAYER:" "$TEMP1"

ORIG_X=`echo "$TEMP1"|grep ID_VIDEO_WIDTH|cut -d"=" -f2`
debugmsg "parsed output for ORIG_X:" $ORIG_X

ORIG_Y=`echo "$TEMP1"|grep ID_VIDEO_HEIGHT|cut -d"=" -f2`
debugmsg "parsed output for ORIG_Y:" $ORIG_Y

ORIG_FPS=`echo "$TEMP1"|grep ID_VIDEO_FPS|cut -d"=" -f2`
debugmsg "parsed output for ORIG_FPS:" $ORIG_FPS
test $ORIG_FPS == "14.985" && ORIG_FPS=29.970 && debugmsg "set ORIG_FPS to:" $ORIG_FPS
test $ORIG_FPS == "15.000" && ORIG_FPS=29.970 && debugmsg "set ORIG_FPS to:" $ORIG_FPS
test $ORIG_FPS == "30.000" && ORIG_FPS=29.970 && debugmsg "set ORIG_FPS to:" $ORIG_FPS

ORIG_ASPECT=`echo "$TEMP1"|grep ID_VIDEO_ASPECT|cut -d"=" -f2`
debugmsg "parsed output for ORIG_ASPECT:" $ORIG_ASPECT

AUDIO_CODEC=`echo "$TEMP1"|grep ID_AUDIO_CODEC|cut -d"=" -f2`
debugmsg "parsed output for AUDIO_CODEC:" $AUDIO_CODEC

return;
}

function calcnewxy () {

local -i TEMP1 MAX_X FACT ANZAHL

if test $ORIG_FPS == "29.970" -o $ORIG_FPS == "24.000" -o $ORIG_FPS == "23.976"; then
  if test $ORIG_X -gt 352 -o $ORIG_Y -gt 240; then
    # 640
    MAX_X=$((4*480/3)) ; debugvar MAX_X $MAX_X
  else
    # 320
    MAX_X=$((4*240/3)) ; debugvar MAX_X $MAX_X
  fi
  NEW_Y=$(($MAX_X*ORIG_Y/$ORIG_X)); debugvar NEW_Y $NEW_Y
  echolog "*** INFO: For Sqare Pixels we would scale to $MAX_X x $NEW_Y ..."
  test $NEW_Y -gt 480 && NEW_Y=480 	# Force FullScreen / Verzichtet auf Aspect Ratio !
else
  if test $ORIG_X -gt 384 -o $ORIG_Y -gt 288; then
    # 768
    MAX_X=$((4*576/3)) ; debugvar MAX_X $MAX_X
  else
    # 384
    MAX_X=$((4*288/3)) ; debugvar MAX_X $MAX_X
  fi
  NEW_Y=$(($MAX_X*ORIG_Y/$ORIG_X)); debugvar NEW_Y $NEW_Y
  echolog "*** INFO: For Sqare Pixels we would scale to $MAX_X x $NEW_Y ..."
  test $NEW_Y -gt 576 && NEW_Y=576 	# Force FullScreen / Verzichtet auf Aspect Ratio !
fi


return;
}



function choosebestx () {

local -i MAX_X INDEX
local XResTEMP

if test $ORIG_FPS == "29.970" -o $ORIG_FPS == "24.000" -o $ORIG_FPS == "23.976"; then
  XResTEMP=(${XResNTSC[*]})
else
  XResTEMP=(${XResPAL[*]})
fi
debugvar XResTEMP "${XResTEMP[*]}"

ANZAHL=${#XResTEMP[*]}
debugvar AnzahlVonXResTEMP $ANZAHL

INDEX=-1
declare repeat=true
until ! $repeat;
    do INDEX=$(($INDEX+1))
    if test ${XResTEMP[$INDEX]} -ge $ORIG_X; then repeat=false; NEW_X=${XResTEMP[$INDEX]}; fi
    if test $INDEX -ge $ANZAHL; then repeat=false; NEW_X=${XResTEMP[$ANZAHL]}; fi
done

debugvar NEW_X $NEW_X

return;
}


# UNUSED
function aspectratio () {
# Direkt aus 2divx uebernommen - noch anpassen :-)

# Aufruf: aspectratio /video0/..../001.vdr
# Ausgabe: in Variable $ASPECT (global)

# Variablen Definitionen
local TEMP1
local -a TEMP2[0]
local -i ANZAHL i

ASPECT=2	# Default Value (4:3) falls nicht bestimmt werden kann !

$ENCEXE "$1" -frames 0 -o temp_weg.avi 2>$WRKTMP/temp_weg.txt >$WRKTMP/temp_weg2.txt
TEMP1=`cat $WRKTMP/temp_weg.txt $WRKTMP/temp_weg2.txt | grep -i -e "aspect"`
# Old: # TEMP1=`$ENCEXE $1 -frames 0 -o temp_weg.avi | grep -i -e "aspect"`
TEMP2=($TEMP1)
ANZAHL=${#TEMP2[*]}
i=-1

while i=i+1 ; test $i -le $((ANZAHL-1)); do
# Achtung geht davon aus, dass die Ausgabe von Mencoder lautet .. ( aspect x ) ..
    if test ${TEMP2[$i]#(} == "aspect" ; then ASPECT=${TEMP2[$((i+1))]%)}; fi
done
# ggf erzeugtes temp_weg.avi wieder entfernen !!!
if test -f $WRKTMP/temp_weg.txt; then rm $WRKTMP/temp_weg.txt; fi
if test -f $WRKTMP/temp_weg2.txt; then rm $WRKTMP/temp_weg2.txt; fi
if test -f $WRKTMP/temp_weg.avi; then rm $WRKTMP/temp_weg.avi; fi

return;
}


function checkforac3 () {
  if test $AUDIO_CODEC == "a52" && $USEAC3; then AOUT="$AC3AOUT"; fi
  return; 
}

function echolog () {
# prints the String on stdout and into /var/log/messages using logger !
  echo "$1"
  logger "$1"
  return;
}


function errorcfg () {
  echolog "*** FATAL: Config Option $1 not found in Config File ... Exiting."
  exit;
}


function debugvar () {
  if $DEBUG; then echolog "*** DEBUG: Variable $1 has value \"$2\""; fi
  return;
}


function debugmsg () {
  if $DEBUG; then echolog "*** DEBUG: $1 \"$2\""; fi
  return;
}


# Begin Main !
#
#
# ---------------------------------------------------------------------------
#
#

declare -i ORIG_X ORIG_Y
declare -i NEW_X NEW_Y
declare ORIG_FPS ORIG_ASPECT AUDIO_CODEC
declare CMDLINE AOUT RESCALE REMOTE
declare FILE="$1"
declare AOUT="-ao mpegpes -afm 9"

echolog "*** Starting mplayer.sh Version $VERSION"

if test -z "$FILE"; then echolog "*** USAGE: mplayer.sh <File_to_be_played>"; exit; fi
if ! test -f "$FILE"; then echolog "*** ERROR: Make sure $FILE exists - otherwise it cannot be played ;-)"; exit; fi

# Check if Config File exists !
#declare CFGFIL="./mplayer.sh.conf"
declare CFGFIL="${0}.conf"
debugvar CFGFIL $CFGFIL
if ! test -f $CFGFIL; then echolog "*** FATAL: mplayer.sh.conf not found !!! Exiting." ; exit; fi

# Debug Mode ?
declare DEBUG=`readcfg debug`
if test -z "$DEBUG"; then DEBUG=false; fi

# use AC3?
declare USEAC3=`readcfg USEAC3`
if test -z "$USEAC3"; then USEAC3=false; fi
debugvar USEAC3 "$USEAC3"

# AC3 command line
declare AC3AOUT=`readcfg AC3AOUT`
if test -z "$AC3AOUT"; then
  errorcfg AC3AOUT
  exit
else
  debugvar AC3AOUT "$AC3AOUT"
fi

# read MPLAYER Config
declare MPLAYER=`readcfg mplayer`
if test -z "$MPLAYER"; then errorcfg MPLAYER; exit; else debugvar MPLAYER "$MPLAYER"; fi

declare VOP=`readcfg vop`
if test -z "$VOP"; then errorcfg VOP; exit; else debugvar VOP "$VOP"; fi

declare CACHESTR="" # no cache
declare CACHE=`readcfg cache`
if ! test -z $CACHE; then CACHESTR="-cache $CACHE"; debugvar CACHE "$CACHE"; debugvar CACHESTR "$CACHESTR";
		     else echolog "*** Option CACHE not set in Config File - calling mplayer without Cache !";
fi

declare FRAMEDROP=`readcfg framedrop`
if test -z "$FRAMEDROP"; then FRAMEDROP=false; fi
debugvar FRAMEDROP "$FRAMEDROP"
declare FDSTR=""
if $FRAMEDROP; then FDSTR="-framedrop"; fi 
debugvar FDSTR "$FDSTR"

declare LIRCSTR="" # no extra Lirc option !
declare LIRCRC=`readcfg lircrc`
if ! test -z "$LIRCRC"; then LIRCSTR="-lircconf $LIRCRC"; fi
debugvar LIRCRC "$LIRCRC"; debugvar LIRCSTR "$LIRCSTR"

if [ "$2" != "SLAVE" ]; then
  REMOTE="$LIRCSTR"
else
  REMOTE="-slave -nolirc"
fi
debugvar REMOTE "$REMOTE"

declare -a USERDEF[0]
declare USERDEF=(`readcfg userdef`)
if ! test -z "$USERDEF"; then echolog "*** Use Option USERDEF at your own risk !"; fi
debugvar USERDEF "${USERDEF[*]}"

declare -a XResPAL[0]
declare XResPAL=(`readcfg XResPAL`)
if test -z $XResPAL; then errorcfg XResPAL; exit; else debugvar XResPAL "${XResPAL[*]}"; fi

declare -a XResNTSC[0]
declare XResNTSC=(`readcfg XResNTSC`)
if test -z $XResNTSC; then errorcfg XResNTSC; exit; else debugvar XResNTSC "${XResNTSC[*]}"; fi

# Try to determine the video attributes

if $DEBUG; then echolog "*** DEBUG: Calling getvidxy function to analyze source video stream ..."; fi
getvidxy
if test $ORIG_X -eq 0 -o $ORIG_Y -eq 0; then echolog "*** FATAL: Illegal Resolution ..."; exit; fi
echolog "*** INFO: Source Video has Resolution of $ORIG_X x $ORIG_Y ..."

checkforac3
calcnewxy
choosebestx
if test $NEW_X -eq 0 -o $NEW_Y -eq 0; then echolog "*** FATAL: Illegal Resolution ..."; exit; fi

case $ORIG_FPS in
  "29.970"|"23.976"|"24.000")
    debugmsg "NTSC"
    if test \( $ORIG_Y -eq 240 -a $ORIG_X -eq 352 \) -o $ORIG_Y -eq 480; then
      RESCALE="-vop $VOP,expand=-1:-1:-1:-1:1,scale=-1:-1"
    else
      if test $ORIG_X -gt 352 -o $ORIG_Y -gt 240; then
        debugmsg "ORIG_Y greater than 240 or ORIG_X grater than 352:" $ORIG_Y
        RESCALE="-vop $VOP,expand=$NEW_X:480:-1:-1:1,scale=$NEW_X:$NEW_Y"
      else
        debugmsg "ORIG_Y less than or equal to 240 or ORIG_X less than or equal to 352:" $ORIG_Y
        RESCALE="-vop $VOP,expand=$NEW_X:240:-1:-1:1,scale=$NEW_X:$NEW_Y"
      fi
    fi
  ;;

  *)
    debugmsg "PAL or unknown"
    if test \( $ORIG_Y -eq 288 -a $ORIG_X -eq 352 \) -o $ORIG_Y -eq 576; then
      RESCALE="-vop $VOP,expand=-1:-1:-1:-1:1,scale=-1:-1"
    else
      if test $ORIG_X -gt 352 -o $ORIG_Y -gt 288; then
        debugmsg "ORIG_Y greater than 288 or ORIG_X grater than 352:" $ORIG_Y
        RESCALE="-vop $VOP,expand=$NEW_X:576:-1:-1:1,scale=$NEW_X:$NEW_Y"
      else
        debugmsg "ORIG_Y less than or equal to 288 or ORIG_X less than or equal to 352:" $ORIG_Y
        RESCALE="-vop $VOP,expand=$NEW_X:288:-1:-1:1,scale=$NEW_X:$NEW_Y"
      fi
    fi
  ;;
esac
CMDLINE="$MPLAYER -vo mpegpes $AOUT $RESCALE $FDSTR $CACHESTR $REMOTE $USERDEF"
debugvar CMDLINE "$CMDLINE"
exec $CMDLINE "$FILE"
# exec $CMDLINE "$FILE" -
exit
